;
.include "Header.inc"     ;Label und Werte Zuweisungen
.include "Hardware.inc"   ;Hardware Label  Zuweisungen
;
;Programmstart nach RESET  ( Interrupt Vektoren Tabelle )
;
.CSEG                       ;Was ab hier folgt kommt in den FLASH-Speicher
.ORG $0000                  ;Programm beginnt an der FLASH-Adresse 0x0000

 rjmp  _reset               ;RESET-Vector

.include "IRQm8.inc";Restliche Interrupt Vektortabelle

.ORG INT_VECTORS_SIZE       ;Programmadresse nach den ganzen IRQ-Vektoren

;
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;I Erstinitialisierungen
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;
;Stapelzeiger initialisieren (fuer Unterprogramme und / oder Interrupts)
;
_reset:
 ldi  a,High(RAMEND)        ;RAMEND (SRAM) ist in der Definitions-
 out  SPH,a                 ;datei festgelegt  
 ldi  a,Low (RAMEND)
 out  SPL,a
;
;Variablen initialisieren ( r0-r31 haben unbestimmten Inhalt )
;
 clr  key_reg               ;Tasterentprellregister
;
;Ports Erstkonfiguration
; 
 sbi  LED_PORT,led.ok       ;OK-LED ausschalten ( Vorerst Pullup )
 sbi  LED_DDR,led.ok        ;PORT-Bit als Ausgang f. OK-LED
 sbi  KEY_PORT,key.bit      ;Pullup f. Taster einschalten

;
;Timer0 Overflow Interrupt initialisieren
;
 ldi  a,1<<TOIE0            ;Timer0 Overflow Interrupt freigeben...
 out  TIMSK,a               ;...
 ldi  a,1<<CS02             ;Timer0 Vorteiler => 256...
 out  TCCR0,a               ;...
;
;TWI SCL Frequenz einstellen ( ca. 102 kHz mit TWBR=2 ca. 92kHz )
;und Two-wire Interface Interrupt freigeben
 ldi  a,#twbr                ;Ergibt zusammen mit den Vorteilerbits...
 out  TWBR,a                 ;...TWPS=0 im TWSR ca. 102 kHz
 in  a,TWCR                 ;TWI Control Register lesen...
 ori a,1<<TWIE ;|1<<TWEN             ;...und TWI-Interrupt... 
 out TWCR,a                 ;...freigeben

 sei                        ;Globale Interruptfreigabe im CPU-Statusregister
;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;H Hauptprogrammschleife
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;
_main:
 sbrs key_reg,key.actuated  ;Taster betaetigt und entprellt?...
 rjmp _idle                 ;...Nein => Weiter im Progrmam
;
;TWI START-Bedingung mit Interruptfreigabe
;
 rcall _twi_start           ;TWI START-Bedingung mit Interruptfreigabe

 cbr  key_reg,1<<key.actuated;Tasterbetaetigung quitieren

_idle:  
 rjmp _main                 ;Sprung zum Anfang des Hauptprogramms ( Mainloop )
; 
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;U Beginn der verwendeten Unterprogramme
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;
.include "UP.inc";Unterprogramme
;
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;Q Beginn der verwendeten Interrupt Service Routinen ( ISR )
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;
.include "ISR.inc";Interrupt Service Routinen

.EXIT                       ;Ende des Quelltextes
